pre { /* the pre makes the border */
    margin: 0.5em 0;
    padding: 10pt;
    border: 1pt solid #AEBDCC;
    background-color: #F3F5F7;
    color: black;
}

pre code { /* the code has no border and is inside the pre */
    margin: 0;
    padding: 0;
    border: none;
}

